#!/bin/bash

# Use:
# upos.sh init
# upos.sh ping dir_out prefix
# upos.sh closeday dir_out prefix
# upos.sh pay amount dir_out prefix
# upos.sh bio amount photopath dir_out prefix

# Коды выхода
# 0 - успех
# 1 - устройство не найдено / не отвечает / ошибка выполнение операции, нужно смотреть *e
# 2 - нет исполняемого sb_pilot
# 3 - неверное количество аргументов
# 4 - неизвестное действие
# 5 - отсутствует директория для сохранения результата
# 6 - не создались файлы данных sb_pilot

# Ищет утилиту sb_pilot
function search_pilot() {
    HOME=~

    # Ищем утилитку через which
    APP=sb_pilot
    WH=$(which $APP 2>&1)
    if [[ $? -eq 0 ]]; then
        SBP=$WH
    fi

    # Пути поска утилит. Нужно, так как в разных ISO утилиты лежат в разных папках
    if [[ -z "${SBP}" ]]; then
        apppaths=(
            "$HOME/atm/bin/pos/upos/$APP"
            "$HOME/application/bin/upos/$APP"
            "$HOME/postamat/bin/pos/upos/$APP"
        )

        for path in "${apppaths[@]}" ; do
            if [ -f $path ]; then
                SBP=$path
                break
            fi
        done
    fi

    if [[ -z "${SBP}" ]]; then
        echo "Sber cli is not found"
        exit 2
    fi

    SBPATH="$(dirname $(realpath $SBP))"
    ERROR_FILE="$SBPATH/e"
    PRINT_FILE="$SBPATH/p"
    rm -f $ERROR_FILE
    rm -f $PRINT_FILE
}

# Получает статус завершения операции
function sl_get_status() {
    if [[ -f ${ERROR_FILE} ]]; then
        head -n1 ${ERROR_FILE} | cut -f1 -d","
    else
        echo "File ${ERROR_FILE} is not exists"
        exit 6
    fi
}

function sl_set_result() {
    FN="${1}"

    if [[ -f ${ERROR_FILE} ]]; then
        echo "TermNum=`sed "8q;d" ${ERROR_FILE}`" > ${FN}_p
        echo "ClientCard=`sed "2q;d" ${ERROR_FILE}`" >> ${FN}_p
        echo "ClientExpiryDate=`sed "3q;d" ${ERROR_FILE}`" >> ${FN}_p
        echo "AuthCode=`sed "4q;d" ${ERROR_FILE}`" >> ${FN}_p
        echo "CardName=`sed "6q;d" ${ERROR_FILE}`" >> ${FN}_p
        echo "CardType=`sed "7q;d" ${ERROR_FILE}`" >> ${FN}_p
        echo "MerchNum=`sed "14q;d" ${ERROR_FILE}`" >> ${FN}_p
        echo "RRN=`sed "10q;d" ${ERROR_FILE}`" >> ${FN}_p
        echo "TrxDate=`sed "9q;d" ${ERROR_FILE}`" >> ${FN}_p
        echo "Hash=`sed "11q;d" ${ERROR_FILE}`" >> ${FN}_p
        head -n1 ${ERROR_FILE} | cut -f1 -d"," > ${FN}_e
    fi

    if [[ -f ${PRINT_FILE} ]]; then
        cat ${PRINT_FILE} > ${FN}_c
    fi
}

function sl_set_status() {
    if [[ -f ${ERROR_FILE} ]]; then
        echo "${1}" > "${2}_e"
    else
        echo "File ${ERROR_FILE} is not exists"
        exit 6
    fi
    if [ "${1}" = "0" ]; then
        exit 0
    else
        exit 1
    fi
}


# Проверяет количество аргументов в команде
function arg_count() {
    if [[ $1 -ne $2 ]]; then
        echo "Wrong argument count"
        exit 3
    fi
}

function sl_init() {
    arg_count 1 $#
    rm -f /run/lock/sbrf-*

    # Новые модели пос-терминалов добавлять сюда построчно
    sberdev=(
        usb-upos usb-079b_0028-if00
        usb-VeriFone_Inc_Trident_USB_Device_1.1_0123456789ABCD-if00
        usb-VeriFone_Verix_USB_Device_323-277-3449-if00
        usb-PAX_d200-if00
        usb-PAX_S300-if00
    )

    initres="1"
    for device in $(ls /dev/serial/by-id/) ; do
        for sb in "${sberdev[@]}" ; do
            if [ "${device,,}" = "${sb,,}" ]; then
              rm -f ${SBPATH}/ttyS99
              ls -al /dev/serial/by-id/${device}
              ln -sf /dev/serial/by-id/${device} ${SBPATH}/ttyS99
              ls -al ${SBPATH}/ttyS99
              initres="0"
            fi
        done
    done
}

# Конвертирует файлы сбера в кодировку utf + меняет формат переводов кареток в unix-формат
function sl_to_utf() {
    if [ -f "$ERROR_FILE" ]; then
        iconv -f koi8-r "$ERROR_FILE" > "$ERROR_FILE.new" && mv -f "$ERROR_FILE.new" "$ERROR_FILE"
        dos2unix "$ERROR_FILE" > /dev/null 2>&1
    fi
    if [ -f "$PRINT_FILE" ]; then
        iconv -f koi8-r "$PRINT_FILE" > "$PRINT_FILE.new" && mv -f "$PRINT_FILE.new" "$PRINT_FILE"
        dos2unix "$PRINT_FILE" > /dev/null 2>&1
    fi
}

function sl_sbp_exec() {
    cd "${SBPATH}" && ${SBP} "$@" > /dev/null 2>&1
    sl_to_utf
}

function sl_sbp_process() {
    res=$(sl_get_status)
    sl_set_result "${2}/${1}_${3}"
    sl_set_status ${res} "${2}/${1}_${3}"
}

function sl_ping() {
    arg_count 3 $#
    sl_sbp_exec 26 0
    sl_sbp_process ${1} ${2} ${3}
}

function sl_close() {
    arg_count 3 $#
    sl_sbp_exec 7
    sl_sbp_process ${1} ${2} ${3}
}

function sl_paycard() {
    arg_count 4 $#
    sl_sbp_exec 1 ${2} 0
    sl_sbp_process ${1} ${3} ${4}
}

function sl_paybio() {
    arg_count 5 $#
    sl_sbp_exec 39 ${2} /i=${3} 0
    sl_sbp_process ${1} ${4} ${5}
}

function sl_info() {
    arg_count 3 $#
    sl_sbp_exec 36
    sl_sbp_process ${1} ${2} ${3}
}

function sl_unknown(){
    echo "Unknown function $ACTION"
    exit 4
}

################################################################################
# Как минимум должен быть 1 параметр с типом действия
if [[ -z "${1}" ]]; then
    echo "Action is not set"
    exit 3
fi

# Ищем пилот
search_pilot


# Если нашли - обрабатываем запрос
case "${1}" in
    ping     ) sl_ping "$@" ;;
    init     ) sl_init "$@" ;;
    closeday ) sl_close "$@" ;;
    pay      ) sl_paycard "$@" ;;
    bio      ) sl_paybio "$@" ;;
    info     ) sl_info "$@" ;;
    *        ) sl_unknown ;;
esac

exit 0
